;
; Checks the documentation for various errors

#Region Includes
#include "Array.au3"
#include "FileConstants.au3"
#include "include\OutputLib.au3"
#include "include\SettingsLib.au3"
#EndRegion Includes

#Region Global Variables
; The name of the project.
Global Const $g_sProject = "Check Includes libExamples .au3"
Global Const $g_sProjectDir = "docs\autoit\english\txt2htm\libExamples"

#EndRegion Global Variables

#Region Main body of code
Global $g_nExitCode = Main($g_sProject)
Exit $g_nExitCode
#EndRegion Main body of code

#Region Main()
; ===================================================================
; Main()
;
; The main program body.
; Parameters:
;	None.
; Returns:
;	None.
; ===================================================================
Func Main($sProject)
	Local $nReturn = 0

	; Create the output window and initial message.
	_OutputWindowCreate()
	_OutputProgressWrite("==== Output for " & StringTrimRight(@ScriptName, StringLen(".exe")) & " (" & $sProject & ") ====" & @CRLF)

	; Set the build directory based on the rules and the INI file value.
	_SettingBuildDir()

	; Get Functions from Include Files.
	Local $aFunctions[0][2], $aNoDocFunctions[0][2]
	Get_FunctionsInclude("install\include", $aFunctions, $aNoDocFunctions)
	Get_FunctionsInclude("docs\autoit\english\txt2htm\libExamples\Extras", $aFunctions, $aNoDocFunctions)
;~ 	_ArrayDisplay($aFunctions, "$aFunctions")
;~ 	_ArrayDisplay($aNoDocFunctions, "$aNoDocFunctions")

	$nReturn = Verify_IncludesLibExamples($g_sProjectDir, $aFunctions, $aNoDocFunctions)
	ToolTip("")

	; Check the return value.
	If $nReturn Then
		_OutputProgressWrite("Checking failed (" & $nReturn & ")." & @CRLF)
	Else
		_OutputProgressWrite("Checking complete." & @CRLF)
	EndIf

	; Write closing message and wait for close (if applicable).
	_OutputProgressWrite("Finished." & @CRLF & @CRLF) ; Two CRLF's in case of chained output.
	_OutputWaitClosed($nReturn)

	; Return the value.
	Return $nReturn
EndFunc   ;==>Main
#EndRegion Main()

#Region Primary Functions

#Region Get_FunctionsInclude()

Func Get_FunctionsInclude($sIncludeDir, ByRef $aFunctions, ByRef $aNoDocFunctions)
	Local $sFile, $sIncludeFile
	Local $hSearch = FileFindFirstFile($sIncludeDir & "\*.au3")
	While 1
		$sFile = FileFindNextFile($hSearch)
		If @error Then ExitLoop
		If StringInStr($sFile, "Constants") Then ContinueLoop
		If StringInStr($sFile, "SQLite.dll") Then ContinueLoop

		$sIncludeFile = FileRead($sIncludeDir & "\" & $sFile)
		_ArrayConcatenate($aNoDocFunctions, GetNoDocFunctions($sIncludeFile, $sFile))
;~ 		If @error Then
;~ 			_OutputBuildWrite("No N0_DOC section in " & $sFile & @CRLF)
;~ 		EndIf
		_ArrayConcatenate($aFunctions, GetCurrentFunctions($sIncludeFile, $sFile))
;~ 		If @error Then
;~ 			_OutputBuildWrite("No CURRENT section in " & $sFile & @CRLF)
;~ 		EndIf
	WEnd

	; Close the search handle
	FileClose($hSearch)

EndFunc   ;==>Get_FunctionsInclude

#EndRegion Get_FunctionsInclude()

#Region Verify_IncludesLibExamples()

Func Verify_IncludesLibExamples($sProjectDir, ByRef $aFunctions, ByRef $aNoDocFunctions)
	_OutputProgressWrite("Includes LibExamples verification ..." & @CRLF)
	Local $sFile, $sExample, $aInclude, $aConstantsInclude, $sConstantsInclude, $aUDFs, $aInternalFunc, $nReturn = 0, $i, $vRange
	Local $hSearch = FileFindFirstFile($sProjectDir & "\*.au3")
	While 1
		$sFile = FileFindNextFile($hSearch)
		If @error Then ExitLoop
		ToolTip($sFile, 200, 200)

		$sExample = FileRead($sProjectDir & "\" & $sFile)
		$aInternalFunc = StringRegExp($sExample, "Func (.*?)\(", $STR_REGEXPARRAYGLOBALMATCH)
		For $i = 0 To UBound($aInternalFunc) - 1
			$sExample = StringRegExpReplace($sExample, '(?<!\$|\w)' & $aInternalFunc[$i] & '(?=\h*\()', '')
		Next

		$sExample = StringRegExpReplace($sExample, '''(.*?)''', '')
		$sExample = StringRegExpReplace($sExample, '"(.*?)"', '')

		$aInclude = StringRegExp($sExample, "#[iI]nclude <(.*?)>", $STR_REGEXPARRAYGLOBALMATCH)

		; keep only Constants Includes
		$aConstantsInclude = $aInclude
		$vRange = ""
		$i = -1
		For $sConstantsInclude In $aConstantsInclude
			$i += 1
			If StringInStr($sConstantsInclude, "Constants") Then ContinueLoop
			If StringInStr($sConstantsInclude, "SQLite.dll") Then ContinueLoop
			If StringInStr($sConstantsInclude, "WMDebug") Then ContinueLoop
			If StringInStr($sConstantsInclude, "WM_NOTIFY") Then ContinueLoop
			$vRange &= $i & ";"
		Next
		If $vRange Then
			$vRange = StringTrimRight($vRange, 1)
			_ArrayDelete($aConstantsInclude, $vRange)
		EndIf

		$aUDFs = StringRegExp($sExample, '(_[\w\d]*?)\(', $STR_REGEXPARRAYGLOBALMATCH)
		$aUDFs = _ArrayUnique($aUDFs, 0, 0, 0, $ARRAYUNIQUE_NOCOUNT)

		If Not Verify_Includes($aInclude, $aUDFs, $aFunctions, $aNoDocFunctions) Then
			$nReturn += 1
			_OutputBuildWrite($nReturn & " Wrong Includes in " & $sFile & @CRLF & @CRLF)
		EndIf

		Update_Includes($aInclude, $aConstantsInclude, $sProjectDir & "\" & $sFile)

;~ 		If $nReturn = 1 Then
;~ 			_OutputWaitClosed($nReturn)
;~ 			Exit
;~ 		EndIf
	WEnd

	; Close the search handle
	FileClose($hSearch)

	Return $nReturn
EndFunc   ;==>Verify_IncludesLibExamples

#EndRegion Verify_IncludesLibExamples()

#Region Update_Includes()

Func Update_Includes(ByRef $aInclude, ByRef $aConstantsInclude, $sFile)
	Local $sInclude, $aNewInclude = $aConstantsInclude
	_ArrayAdd($aNewInclude, $aInclude)
	$aNewInclude = _ArrayUnique($aNewInclude, 0, 0, 0, $ARRAYUNIQUE_NOCOUNT)
	_ArraySort($aNewInclude)

	Local $sLine, $hFile = FileOpen($sFile)
	Local $bCheckIncludes = True
	Local $sTemp = @TempDir & "\updateExample.au3", $hTemp = FileOpen($sTemp, $FO_OVERWRITE)
	While 1
		$sLine = FileReadLine($hFile)
		If @error Then ExitLoop
		If $bCheckIncludes And (StringLeft($sLine, 10) = "#include <") Then
			$bCheckIncludes = False
			While (StringLeft($sLine, 10) = "#include <") Or ($sLine = "")
				$sLine = FileReadLine($hFile)
			WEnd
			For $sInclude In $aNewInclude
				If $sInclude Then FileWriteLine($hTemp, "#include <" & $sInclude & ">")
			Next
			FileWriteLine($hTemp, "")
		EndIf
		FileWriteLine($hTemp, $sLine)
	WEnd
	FileClose($hTemp)
	FileClose($hFile)

	If FileRead($sFile) <> FileRead($sTemp) Then
		ConsoleWrite('@@ Debug(' & @ScriptLineNumber & ') : $sFile = ' & $sFile & @CRLF & '>Error code: ' & @error & '    Extended code: ' & @extended & ' (0x' & Hex(@extended) & ')' & @CRLF) ;### Debug Console
		FileCopy($sTemp, $sFile, $FC_OVERWRITE)
		$sFile = StringMid($sFile, StringInStr($sFile, "\", 0, -1) + 1)
		_OutputBuildWrite($sFile & @TAB & "has been updated" & @CRLF)
	EndIf
	FileDelete($sTemp)
EndFunc   ;==>Update_Includes

#EndRegion Update_Includes()

#Region Verify_Includes()

Func Verify_Includes(ByRef $aInclude, ByRef $aUDFs, ByRef $aFunctions, ByRef $aNoDocFunctions)
	Local $nReturn = 0, $sNewInclude, $aNewInclude[0], $sNoDocFunction
	For $sUDF In $aUDFs
		$sNewInclude = Get_IncludeName($sUDF, $aFunctions)
		If $sNewInclude Then
			_ArrayAdd($aNewInclude, $sNewInclude)
			For $sInclude In $aInclude
				If $sInclude = $sNewInclude Then ContinueLoop 2
			Next
			$nReturn += 1
			_OutputBuildWrite($nReturn & " " & $sUDF & "() need #Include " & $sNewInclude & @CRLF)
		Else
			$sNoDocFunction = Get_IncludeName($sUDF, $aNoDocFunctions)
			If $sNoDocFunction = "" Then
				_OutputProgressWrite($sUDF & "() is a NO_DOC_FUNCTION" & @CRLF)
			EndIf
		EndIf
	Next
	$aInclude = _ArrayUnique($aNewInclude, 0, 0, 0, $ARRAYUNIQUE_NOCOUNT)

	Return ($nReturn = 0)
EndFunc   ;==>Verify_Includes

#EndRegion Verify_Includes()

#Region Get_IncludeName()

Func Get_IncludeName($sFunctions, ByRef $aFunctions)
	For $i = 0 To UBound($aFunctions) - 1
		If $sFunctions = $aFunctions[$i][0] Then Return $aFunctions[$i][1]
	Next
	Return ""
EndFunc   ;==>Get_IncludeName

#EndRegion Get_IncludeName()

#Region GetCurrentFunctions()

Func GetCurrentFunctions($sIncludeFile, $sFile)
	Local $aFiles, $aFilesInclude
	Local $j, $i = StringInStr($sIncludeFile, "; #CURRENT# ="), $sCurrentText
	If $i Then
		$i = StringInStr($sIncludeFile, ";", 0, 2, $i)
		$j = StringInStr($sIncludeFile, "; ===", 0, 1, $i)
		$sCurrentText = StringMid($sIncludeFile, $i, $j - $i)
		$sCurrentText = StringReplace($sCurrentText, @TAB, " ")
		$aFiles = StringRegExp(StringMid($sIncludeFile, $i, $j - $i), "; (.*)\r\n", $STR_REGEXPARRAYGLOBALMATCH)
		Dim $aFilesInclude[UBound($aFiles)][2]

		UpdateIncludeFile($aFilesInclude, $aFiles, $sFile)

	EndIf
	Return $aFilesInclude
EndFunc   ;==>GetCurrentFunctions

Func GetNoDocFunctions($sIncludeFile, $sFile)
	Local $aFiles, $aFilesInclude
	Local $j, $i = StringInStr($sIncludeFile, "; #NO_DOC_FUNCTION# ="), $sCurrentText
	If $i Then
		$i = StringInStr($sIncludeFile, ";", 0, 3, $i)
		$j = StringInStr($sIncludeFile, "; ===", 0, 1, $i)
		$sCurrentText = StringMid($sIncludeFile, $i, $j - $i)
		$sCurrentText = StringReplace($sCurrentText, @TAB, " ")
		$aFiles = StringRegExp(StringMid($sIncludeFile, $i, $j - $i), "; (.*)\r\n", $STR_REGEXPARRAYGLOBALMATCH)

		Dim $aFilesInclude[UBound($aFiles)][2]

		UpdateIncludeFile($aFilesInclude, $aFiles, $sFile)

	EndIf
	Return $aFilesInclude
EndFunc   ;==>GetNoDocFunctions

#EndRegion GetCurrentFunctions()

#Region UpdateIncludeFile()

Func UpdateIncludeFile(ByRef $aFilesInclude, ByRef $aFiles, $sFile)
	If Not IsArray($aFiles) Then Return
	Local $k = 0, $vRange = "", $sCurInclude = $sFile, $sFunctionName
	For $i = 0 To UBound($aFiles) - 1
		$sFunctionName = $aFiles[$i]
		If StringLeft($sFunctionName, 6) = "Doc in" Then
			$vRange &= $k & ";"
			$sCurInclude = StringTrimLeft($sFunctionName, 7) & ".au3"
		ElseIf StringLeft($sFunctionName, 1) <> "_" Then
			$vRange &= $k & ";"
		Else
			$aFilesInclude[$i][0] = $sFunctionName
			$aFilesInclude[$i][1] = $sCurInclude
		EndIf
		$k += 1
	Next
	If $vRange Then
		$vRange = StringTrimRight($vRange, 1)
		_ArrayDelete($aFilesInclude, $vRange)
	EndIf
	If UBound($aFilesInclude) = 0 Then $aFilesInclude = ""
EndFunc   ;==>UpdateIncludeFile

#EndRegion UpdateIncludeFile()

#EndRegion Primary Functions
