;
; Builds UDFs3 help file
;

#Region Includes
#include "include\OutputLib.au3"
#include "include\SettingsLib.au3"
#EndRegion Includes

#Region Global Variables
; The name of the project.
Global Const $g_sProjectLang = "english"
Global Const $g_sProject = "UDFs3 help"
Global Const $g_sProjectDir = "docs\autoit"
#EndRegion Global Variables

#Region Main body of code
Global $g_nExitCode = _Main()
Exit $g_nExitCode
#EndRegion Main body of code

#Region _Main()
; ===================================================================
; _Main()
;
; The main function responsible for generating the syntax files.
; Parameters:
;	None.
; Returns:
;	0 on success, non-zero on failure.
; ===================================================================
Func _Main()
	; Create the output window and initial message.
	_OutputWindowCreate()
	_OutputProgressWrite("==== Output for " & StringTrimRight(@ScriptName, StringLen(".exe")) & " (" & $g_sProject & ") ====" & @CRLF)
	_OutputProgressWrite("Generating UDFs .htm..." & @CRLF)

	; Set the build directory based on the rules and the INI file value.
	Local $sBuildDir = _SettingBuildDir()

	; Get Setting to check if Rebuild all help files is required
	Local $sRegenAll = ""
	If _SettingGet($SETTING_REBUILDHELPFILES, False, True, Default, True) Then $sRegenAll = "/RegenAll"

	; Update the helpfile
	FileChangeDir($sBuildDir & "\" & $g_sProjectDir & "\" & $g_sProjectLang)
	RunWait('"' & @AutoItExe & '" All_Gen_UDFs3.au3 ' & $sRegenAll)

	; Holds the return value.
	Local $iReturn = 0

	; Delete all temp files ready for source code packaging
	FileDelete($g_sProjectDir & "\" & $g_sProjectLang & "\Debug.log")
	FileDelete($g_sProjectDir & "\" & $g_sProjectLang & "\_errorlogUDF3.txt")
	FileDelete($g_sProjectDir & "\" & $g_sProjectLang & "\genindex.log")

	; Write closing message and wait for close (if applicable).
	_OutputProgressWrite("Finished." & @CRLF & @CRLF) ; Two CRLF's in case of chained output.

	_OutputWaitClosed($iReturn)

	Return $iReturn
EndFunc   ;==>_Main
#EndRegion _Main()
