; ===================================================================
; Project: SciTE Library
; Description: This file contains SciTE related functions.
; Author: guinness
; ===================================================================
#include-once

#include <SendMessage.au3>
#include <StringConstants.au3>

#include 'MiscLib.au3'
#include 'OutputLib.au3' ; $tagCOPYDATASTRUCT

Global Const $SCITE_GUID = '852976AB-91E5-4934-A64D-340CE3F9C968', _
		$SCITE_DIRECTORTITLECLASS = 'DirectorExtension', _
		$SCITE_EDITTITLECLASS = '[CLASS:Scintilla; INSTANCE:1]', _
		$SCITE_SESSIONPATH = _WinAPI_GetTempFileName(@TempDir), _
		$SCITE_TITLECLASS = '[CLASS:SciTEWindow]'
Global $g_sSCITE_FILEPATH = _GetFullPath('..\..\..\install\SciTe\SciTE.exe'), _
		$g_sSCITE_KEYWORDSPROPERTIESPATH = _GetFullPath('..\..\..\install\SciTe\au3.keywords.properties')

Global Const $EXIT_ERROR_INVALIDSCITE = 1001
Global Enum $SCITE_USERWND, $SCITE_DIRECTOR, $SCITE_WND, $SCITE_EDIT, $SCITE_DATA, $SCITE_MAX
Global $g_aSciTEAPI[$SCITE_MAX] ; SciTE related array.

; Clear the internal array contents.
_SciTE_Shutdown()

; Close SciTE when the process closes.
OnAutoItExitRegister(_SciTE_Shutdown)

Func _SciTE_AskFileName()
	__SciTE_Send_Command(_SciTE_GetUserWnd(), _SciTE_GetDirector(), 'askfilename:')
	Local $sData = _SciTE_GetData()
	__SciTE_StripProperties($sData) ; Strip the property portion of the return data.
	Return $sData
EndFunc   ;==>_SciTE_AskFileName

Func _SciTE_Au3ToHTML($sFilePath, $sTempFilePath, $bExpandLinks = True)
	_SciTE_ExportToHTML($sFilePath, $sTempFilePath)
	Local $sData = FileRead($sTempFilePath)
	If _SciTE_ParseHTML($sData) And $bExpandLinks Then ; Parameter passed as ByRef.
		; Add Links to known keywords
		; UDF
		$sData = StringRegExpReplace($sData, '<span class="S15">([\w]+?)</span>', '<a class="codeSnippetLink" href="../libfunctions/\1.htm"><span class="S15">\1</span></a>')
		; LIBFUNC-structure of $tag ... in the examples
		$sData = StringRegExpReplace($sData, '<span class="S9">(\$tag\w+?)</span>', '<a class="codeSnippetLink" href="\1.htm"><span class="S9">\1</span></a>')
		; Functions
		$sData = StringRegExpReplace($sData, '<span class="S4">([\w]+?)</span>', '<a class="codeSnippetLink" href="../functions/\1.htm"><span class="S4">\1</span></a>')
		$sData = StringReplace($sData, 'href="../functions/Opt.htm">', 'href="../functions/AutoItSetOption.htm">')
		; Exception for UDPStartup(), UDPShutdown()
		$sData = StringReplace($sData, '<a class="codeSnippetLink" href="UDPStartup.htm"><span class="S4">UDPStartup</span></a>', '<a class="codeSnippetLink" href="TCPStartup.htm"><span class="S4">UDPStartup</span></a>')
		$sData = StringReplace($sData, '<a class="codeSnippetLink" href="UDPShutdown.htm"><span class="S4">UDPShutdown</span></a>', '<a class="codeSnippetLink" href="TCPShutdown.htm"><span class="S4">UDPShutdown</span></a>')
		; Macros
		$sData = StringRegExpReplace($sData, '(?i)<span class="S6">(@[^<]+)</span>', '<a class="codeSnippetLink" href="../macros.htm#\1"><span class="S6">\1</span></a>')
		; Operators
		$sData = StringRegExpReplace($sData, '(?i)<span class="S8">((?:[+^*/=&-]|&gt;|&lt;)+)</span>', '<a class="codeSnippetLink" href="../intro/lang_operators.htm"><span class="S8">\1</span></a>')
		; Keywords
		$sData = StringRegExpReplace($sData, '(?i)<span class="S5">(Not|And|Or)</span>', '<a class="codeSnippetLink" href="../intro/lang_operators.htm"><span class="S5">\1</span></a>')
		$sData = StringRegExpReplace($sData, '(?i)<span class="S5">(ContinueCase|ContinueLoop|Default|Dim|Do|Enum|Exit|ExitLoop|For|Func|If|ReDim|Select|Static|Switch|While|With)</span>', '<a class="codeSnippetLink" href="../keywords/\1.htm"><span class="S5">\1</span></a>')
		$sData = StringRegExpReplace($sData, '(?i)<span class="S5">(Else|Then|ElseIf|EndIf)</span>', '<a class="codeSnippetLink" href="../keywords/IfElseEndIf.htm"><span class="S5">\1</span></a>')
		$sData = StringRegExpReplace($sData, '(?i)<span class="S5">(Next|To|Step)</span>', '<a class="codeSnippetLink" href="../keywords/For.htm"><span class="S5">\1</span></a>')
		$sData = StringRegExpReplace($sData, '(?i)<span class="S5">(Case|EndSwitch)</span>', '<a class="codeSnippetLink" href="../keywords/Switch.htm"><span class="S5">\1</span></a>')
		$sData = StringRegExpReplace($sData, '(?i)<span class="S5">(Global|Local|Const)</span>', '<a class="codeSnippetLink" href="../keywords/Dim.htm"><span class="S5">\1</span></a>')
		$sData = StringRegExpReplace($sData, '(?i)<span class="S5">(EndFunc|ByRef|Return)</span>', '<a class="codeSnippetLink" href="../keywords/Func.htm"><span class="S5">\1</span></a>')
		$sData = StringRegExpReplace($sData, '(?i)<span class="S5">(True|False)</span>', '<a class="codeSnippetLink" href="../keywords/Booleans.htm"><span class="S5">\1</span></a>')
		$sData = StringReplace($sData, '<span class="S5">Until</span>', '<a class="codeSnippetLink" href="../keywords/Do.htm"><span class="S5">Until</span></a>')
		$sData = StringReplace($sData, '<span class="S5">WEnd</span>', '<a class="codeSnippetLink" href="../keywords/While.htm"><span class="S5">WEnd</span></a>')
		$sData = StringReplace($sData, '<span class="S5">EndSelect</span>', '<a class="codeSnippetLink" href="../keywords/Select.htm"><span class="S5">EndSelect</span></a>')
		$sData = StringReplace($sData, '<span class="S5">In</span>', '<a class="codeSnippetLink" href="../keywords/ForInNext.htm"><span class="S5">In</span></a>')
		$sData = StringReplace($sData, '<span class="S5">EndWith</span>', '<a class="codeSnippetLink" href="../keywords/With.htm"><span class="S5">EndWith</span></a>')
		; Directives
		$sData = StringReplace($sData, '<span class="S11">#OnAutoItStartRegister</span>', '<a class="codeSnippetLink" href="../keywords/OnAutoItStartRegister.htm"><span class="S11">#OnAutoItStartRegister</span></a>')
		$sData = StringReplace($sData, '<span class="S11">#include</span>', '<a class="codeSnippetLink" href="../keywords/include.htm"><span class="S11">#include</span></a>')
		$sData = StringReplace($sData, '<span class="S11">#include-once</span>', '<a class="codeSnippetLink" href="../keywords/include-once.htm"><span class="S11">#include-once</span></a>')
		$sData = StringReplace($sData, '<span class="S11">#RequireAdmin</span>', '<a class="codeSnippetLink" href="../keywords/RequireAdmin.htm"><span class="S11">#RequireAdmin</span></a>')
		$sData = StringReplace($sData, '<span class="S11">#NoTrayIcon</span>', '<a class="codeSnippetLink" href="../keywords/NoTrayIcon.htm"><span class="S11">#NoTrayIcon</span></a>')
	EndIf
	Return $sData
EndFunc   ;==>_SciTE_Au3ToHTML

Func _SciTE_Close()
	Return __SciTE_Send_Command(0, _SciTE_GetDirector(), 'close:')
EndFunc   ;==>_SciTE_Close

Func _SciTE_CloseAll()
	Local $iBuffersOpen = 0
	; Close all open files in SciTE.
	While Not (_SciTE_AskFileName() == '') ; Whilst there is a filepath loaded. I could also get the total number of buffers from the user's property file and just close however many they have at any given time.
		_SciTE_Close() ; Close the file in SciTE.
		$iBuffersOpen += 1
	WEnd
	Return $iBuffersOpen
EndFunc   ;==>_SciTE_CloseAll

Func _SciTE_ExportToHTML($sSource, $sDestination)
	Local $bReturn = FileExists($sSource) > 0
	If $bReturn Then
		If __SciTE_Send_Command(0, _SciTE_GetDirector(), 'open:' & StringReplace($sSource, '\', '\\') & '') Then
			$bReturn = __SciTE_Send_Command(0, _SciTE_GetDirector(), 'exportashtml:' & StringReplace($sDestination, '\', '\\')) ; If true assume the export was successful.
			__SciTE_Send_Command(0, _SciTE_GetDirector(), 'close:')
		EndIf
	EndIf
	Return $bReturn
EndFunc   ;==>_SciTE_ExportToHTML

Func _SciTE_GetDirector($bIsReset = False)
	If $bIsReset Or Not IsHWnd($g_aSciTEAPI[$SCITE_DIRECTOR]) Then
		$g_aSciTEAPI[$SCITE_DIRECTOR] = WinGetHandle($SCITE_DIRECTORTITLECLASS)
	EndIf
	Return $g_aSciTEAPI[$SCITE_DIRECTOR]
EndFunc   ;==>_SciTE_GetDirector

Func _SciTE_GetEditCtrl($bIsReset = False)
	If $bIsReset Or Not IsHWnd($g_aSciTEAPI[$SCITE_EDIT]) Then
		$g_aSciTEAPI[$SCITE_EDIT] = ControlGetHandle(_SciTE_GetSciTE(), '', $SCITE_EDITTITLECLASS)
	EndIf
	Return $g_aSciTEAPI[$SCITE_EDIT]
EndFunc   ;==>_SciTE_GetEditCtrl

Func _SciTE_GetData()
	Local $sData = $g_aSciTEAPI[$SCITE_DATA]
	$g_aSciTEAPI[$SCITE_DATA] = Null ; Reset the data to Null.
	Return $sData
EndFunc   ;==>_SciTE_GetData

Func _SciTE_GetFilePath()
	Return $g_sSCITE_FILEPATH
EndFunc   ;==>_SciTE_GetFilePath

Func _SciTE_GetKeywordProperties()
	Return $g_sSCITE_KEYWORDSPROPERTIESPATH
EndFunc   ;==>_SciTE_GetKeywordProperties

Func _SciTE_GetSciTE($bIsReset = False)
	If $bIsReset Or Not IsHWnd($g_aSciTEAPI[$SCITE_WND]) Then
		$g_aSciTEAPI[$SCITE_WND] = WinGetHandle($SCITE_TITLECLASS)
	EndIf
	Return $g_aSciTEAPI[$SCITE_WND]
EndFunc   ;==>_SciTE_GetSciTE

Func _SciTE_GetSessionPath()
	Return $SCITE_SESSIONPATH
EndFunc   ;==>_SciTE_GetSessionPath

Func _SciTE_GetUserWnd()
	Return $g_aSciTEAPI[$SCITE_USERWND]
EndFunc   ;==>_SciTE_GetUserWnd

Func _SciTE_IsExists()
	Return WinExists(_SciTE_GetDirector(True)) > 0 And WinExists(_SciTE_GetSciTE(True)) > 0
EndFunc   ;==>_SciTE_IsExists

Func _SciTE_IsExistsWait($iTimeout = Default)
	Return WinWait($SCITE_TITLECLASS, '', (($iTimeout = Default) ? 0 : Int($iTimeout))) > 0
EndFunc   ;==>_SciTE_IsExistsWait

Func _SciTE_LoadPropertiesFile($sFilePath)
	Local $sAu3Properties = FileRead($sFilePath)
	$sAu3Properties = StringRegExpReplace($sAu3Properties, '(?:\h*\\\h*\R\h*)', ' ') ; Merge continuation lines.
	$sAu3Properties = StringRegExpReplace($sAu3Properties, '(?m:^#\V*$\R)', '') ; Strip comments.
	Local $aProperties = StringSplit($sAu3Properties, @CRLF, BitOR($STR_ENTIRESPLIT, $STR_NOCOUNT))
	For $sProperty In $aProperties
		If __SciTE_IsValidProperty($sProperty) Then ; Check if a valid property.
			; Update the keyword in SciTE.
			__SciTE_Send_Command(_SciTE_GetUserWnd(), _SciTE_GetDirector(), 'property:' & $sProperty)
		EndIf
	Next
	Return True
EndFunc   ;==>_SciTE_LoadPropertiesFile

Func _SciTE_LoadSession($sFilePath)
	Return __SciTE_Send_Command(0, _SciTE_GetDirector(), 'loadsession:' & StringReplace($sFilePath, '\', '\\'))
EndFunc   ;==>_SciTE_LoadSession

Func _SciTE_Open($sFilePath)
	Return __SciTE_Send_Command(0, _SciTE_GetDirector(), 'open:' & StringReplace($sFilePath, '\', '\\'))
EndFunc   ;==>_SciTE_Open

Func _SciTE_Quit()
	Return __SciTE_Send_Command(0, _SciTE_GetDirector(), 'quit:')
EndFunc   ;==>_SciTE_Quit

Func _SciTE_ParseHTML(ByRef $sData)
	Local $bReturn = False
	; Parse the HTML output.
	$sData = StringRegExp($sData & @CRLF & _
			'<body bgcolor="#000000">' & @CRLF & _
			'' & @CRLF & _
			'</body>' & @CRLF, '(?is:<body bgcolor="#[[:xdigit:]]{6}">\R(.*?)\R</body>\R)', $STR_REGEXPARRAYGLOBALMATCH)[0] ; Access the first element, as this will always return something due to having a fallback.
	$sData = StringReplace($sData, '<br />', '')
	; If not blank, then return True.
	$bReturn = Not (StringStripWS($sData, $STR_STRIPALL) == '')
	Return $bReturn
EndFunc   ;==>_SciTE_ParseHTML

Func _SciTE_ReloadProperties()
	Return __SciTE_Send_Command(_SciTE_GetUserWnd(), _SciTE_GetDirector(), 'reloadproperties:')
EndFunc   ;==>_SciTE_ReloadProperties

Func _SciTE_Run() ; Returns if SciTE was running before.
	Local $bIsSciTEExists = _SciTE_IsExists()
	Local $bIsSciTERunningBefore = $bIsSciTEExists
	If Not $bIsSciTEExists Then
		; Use the generated version of SciTE4AutoIt3 when it exists.
		Local $sSciTEProgram = (FileExists($g_sSCITE_FILEPATH) ? $g_sSCITE_FILEPATH : RegRead('HKLM\Software\Microsoft\Windows\CurrentVersion\App Paths\Scite.Exe', ''))

		$bIsSciTEExists = FileExists($sSciTEProgram) > 0
		; When not found, prompt for the SciTE.exe location.
		If Not $bIsSciTEExists Then
			$sSciTEProgram = FileOpenDialog('Couldn''t find SciTE.exe... please select the SciTE executable.', @WorkingDir, 'SciTE (SciTE.exe)', $FD_FILEMUSTEXIST)
			If @error Then ; If SciTE not chosen then exit the build script.
				MsgBox($MB_SYSTEMMODAL, 'SciTE just couldn''t be found', 'Seems there was an error locating SciTE and as such the help file examples can''t be built.') ; Exit the build scripts.
				Exit $EXIT_ERROR_INVALIDSCITE
			EndIf
		EndIf
		; Set the SciTE_HOME environment variable to override the user's.
		; This prevents the user's property files from getting loaded and causing issues.
		EnvSet("SciTE_HOME", @TempDir)
		Run($sSciTEProgram)
		; Set to true or false if SciTE exists.
		$bIsSciTEExists = _SciTE_IsExistsWait(10) ; Wait for SciTE to load with a timeout of 10 seconds. Default is 5 seconds.
		If Not $bIsSciTEExists Then ; SciTE doesn't exist.
			MsgBox($MB_SYSTEMMODAL, 'SciTE just couldn''t be found', 'Seems there was an error locating SciTE and as such the help file examples can''t be built.') ; Exit the build scripts.
			Exit $EXIT_ERROR_INVALIDSCITE
		EndIf
	EndIf

	; Initialise SciTE processing functions.
	_SciTE_Startup()
	; Save a session to a temporary file.
	_SciTE_SaveSession($SCITE_SESSIONPATH)
	; Close the open files.
	_SciTE_CloseAll()

	; Use the generated version of au3.keywords.properties when it exists. Otherwise copy from Program Files\AutoIt3 or Program Files (x86)\AutoIt3.
	Local $sAu3KeywordsProperties = $g_sSCITE_KEYWORDSPROPERTIESPATH
	If Not FileExists($sAu3KeywordsProperties) Then
		; If AutoIt is x64 or the OS is x64 then retrieve C:\Program Files else C:\Program Files (x86).
		Local $sProgramFilesDirX86 = @ProgramFilesDir
		If @AutoItX64 Then $sProgramFilesDirX86 = StringLeft(@ProgramFilesDir, 2) & "\Program Files (x86)"
		If Not FileCopy($sProgramFilesDirX86 & '\AutoIt3\SciTE\au3.keywords.properties', $sAu3KeywordsProperties, $FC_OVERWRITE_CREATE) Then
			FileCopy($sProgramFilesDirX86 & '\AutoIt3\SciTE\Properties\au3.keywords.properties', $sAu3KeywordsProperties, $FC_OVERWRITE_CREATE) ; This is a last resort that it exists.
		EndIf
	EndIf

	; Load the colour scheme.
	_SciTE_LoadPropertiesFile(@ScriptDir & '\au3.properties')
	If FileExists($sAu3KeywordsProperties) Then ; This is a last resort that it exists.
		; Load the Au3 properties file in the install\SciTE directory.
		_SciTE_LoadPropertiesFile($sAu3KeywordsProperties)
	EndIf

	; Set the state of SciTE to minimized.
	WinSetState(_SciTE_GetSciTE(True), "", @SW_MINIMIZE)

	Return $bIsSciTERunningBefore
EndFunc   ;==>_SciTE_Run

Func _SciTE_SaveSession($sFilePath)
	Return __SciTE_Send_Command(0, _SciTE_GetDirector(), 'savesession:' & StringReplace($sFilePath, '\', '\\'))
EndFunc   ;==>_SciTE_SaveSession

Func _SciTE_SetSciTEDir($sDirPath)
	$g_sSCITE_FILEPATH = _GetFullPath($sDirPath & '\SciTE.exe') ; ..\..\..\install\SciTe\SciTE.exe
	$g_sSCITE_KEYWORDSPROPERTIESPATH = _GetFullPath($sDirPath & '\au3.keywords.properties') ; ..\..\..\install\SciTe\au3.keywords.properties
	Return True
EndFunc   ;==>_SciTE_SetSciTEDir

Func _SciTE_SetUserWnd($hWnd)
	$g_aSciTEAPI[$SCITE_USERWND] = $hWnd
	Return True
EndFunc   ;==>_SciTE_SetUserWnd

Func _SciTE_Shutdown()
	Local $hWnd = _SciTE_GetUserWnd()
	If IsHWnd($hWnd) Then
		GUIRegisterMsg($WM_COPYDATA, '')
		; _GUIRegisterMsgEx($WM_COPYDATA, __SciTE_WM_COPYDATA, False)
		GUIDelete($hWnd)
	EndIf
	FileDelete(_SciTE_GetSessionPath())
	For $i = 0 To $SCITE_MAX - 1
		$g_aSciTEAPI[$i] = Null
	Next
	Return True
EndFunc   ;==>_SciTE_Shutdown

Func _SciTE_Startup($hWnd = Null)
	$hWnd = (($hWnd = Null) ? _SciTE_GetUserWnd() : $hWnd) ; If the handle is null, then use the previously set handle (if set) otherwise use the handle passed.
	If Not IsHWnd($hWnd) Then ; If the handle is not a window create a new GUI.
		_SciTE_Shutdown()
		$hWnd = GUICreate($SCITE_GUID & '-SciTE-Interface', 0, 0, -99, -99, $GUI_SS_DEFAULT_GUI, $WS_EX_TOOLWINDOW)
		GUISetState(@SW_SHOWNOACTIVATE, $hWnd)
	EndIf
	_SciTE_SetUserWnd($hWnd)
	GUIRegisterMsg($WM_COPYDATA, __SciTE_WM_COPYDATA)
	; _GUIRegisterMsgEx($WM_COPYDATA, __SciTE_WM_COPYDATA, True)
	Return $hWnd
EndFunc   ;==>_SciTE_Startup

Func __SciTE_IsValidProperty(ByRef $sProperty)
	Return StringRegExp($sProperty, '^[.\w]+=') > 0
EndFunc   ;==>__SciTE_IsValidProperty

Func __SciTE_Send_Command($hWnd, $hSciTE, $sMsg)
	Local $bReturn = False
	If Not (StringStripWS($sMsg, $STR_STRIPALL) == '') Then
		$sMsg = ':' & Dec(StringTrimLeft($hWnd, StringLen('0x'))) & ':' & $sMsg
		Local Const $tBuffer = DllStructCreate('char cdata[' & StringLen($sMsg) + 1 & ']')
		DllStructSetData($tBuffer, 'cdata', $sMsg)
		Local Const $tCOPYDATASTRUCT = DllStructCreate($tagCOPYDATASTRUCT)
		DllStructSetData($tCOPYDATASTRUCT, 'dwData', 0)
		DllStructSetData($tCOPYDATASTRUCT, 'cbData', DllStructGetSize($tBuffer))
		DllStructSetData($tCOPYDATASTRUCT, 'lpData', DllStructGetPtr($tBuffer))
		_SendMessage($hSciTE, _
				$WM_COPYDATA, _
				$hWnd, _
				DllStructGetPtr($tCOPYDATASTRUCT))
		$bReturn = Not @error
	EndIf
	Return $bReturn
EndFunc   ;==>__SciTE_Send_Command

Func __SciTE_StripProperties(ByRef $sData)
	$sData = StringReplace($sData, '\\', '\') ; Escape backslashes.
	$sData = StringRegExpReplace($sData, '(?:^:\d+:(?:\w+:){1,2}?)', '') ; I could use StringReplace() if I wanted to, as it's the decimal value of _SciTE_WinGetUserGUI() and the message sent.
	Return @extended > 0
EndFunc   ;==>__SciTE_StripProperties

Func __SciTE_WM_COPYDATA($hWnd, $iMsg, $wParam, $lParam)
	#forceref $hWnd, $iMsg, $wParam
	Local $tCOPYDATASTRUCT = DllStructCreate($tagCOPYDATASTRUCT, $lParam)
	Local $tBuffer = DllStructCreate('char cdata[' & DllStructGetData($tCOPYDATASTRUCT, 'cbData') + 1 & ']', DllStructGetData($tCOPYDATASTRUCT, 'lpData'))
	$g_aSciTEAPI[$SCITE_DATA] = StringLeft(DllStructGetData($tBuffer, 'cdata'), DllStructGetData($tCOPYDATASTRUCT, 'cbData'))
	Return $GUI_RUNDEFMSG
EndFunc   ;==>__SciTE_WM_COPYDATA
